/*

  in_cube Gamecube Stream Player for Winamp
  by hcs

  includes work by Destop, bero, El Barto, and Rob

*/

// ADX (headered CRI stream)
#include <windows.h>
#include "cube.h"
#include "wamain.h"

// inputfile == NULL means file is already opened, just reload
// return 1 if valid ADX not detected, 0 on success
int InitADXFILE(char * inputfile, CUBEFILE * adx) {
	unsigned char readbuf[4096];
	int l;

	if (inputfile) {
		adx->ch[0].infile=adx->ch[1].infile=INVALID_HANDLE_VALUE;
		adx->ch[0].infile = CreateFile(inputfile,GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,
			OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);

		if (adx->ch[0].infile == INVALID_HANDLE_VALUE) // error opening file
		{
			return 1;
		}
	}

	SetFilePointer(adx->ch[0].infile,0,0,FILE_BEGIN);
	
	ReadFile(adx->ch[0].infile, readbuf, 4096, &l, NULL);

	if(readbuf[0] != 0x80) 
	{
		// not ADX
		if (inputfile) {
			CloseHandle(adx->ch[0].infile);
			adx->ch[0].infile=INVALID_HANDLE_VALUE;
		}
		return 1;
	}

	adx->ch[0].chanstart = get16bit(&readbuf[2])+4;
	adx->ch[1].chanstart = get16bit(&readbuf[2])+4+18;

 	if(memcmp(readbuf+adx->ch[0].chanstart-6,"(c)CRI",6) || l < 4096) 
	{
		// not ADX
		if (inputfile) {
			CloseHandle(adx->ch[0].infile);
			adx->ch[0].infile=INVALID_HANDLE_VALUE;
		}
		return 1;
	}

	adx->NCH = (int)readbuf[7];
	adx->ch[0].sample_rate = get32bit(&readbuf[8]);
	adx->nrsamples = get32bit(&readbuf[12]);
		
	// check version code, set up looping appropriately
	if (get32bit(&readbuf[0x10])==0x01F40300) { // Soul Calibur 2
			if (adx->ch[0].chanstart-6 < 0x2c) adx->ch[0].loop_flag=0; // if header is too small for loop data...
			else {
				adx->ch[0].loop_flag = get32bit(&readbuf[0x18]);
				adx->ch[0].ea = get32bit(&readbuf[0x28]);
				adx->ch[0].sa = get32bit(&readbuf[0x1c])*adx->NCH*18/32+adx->ch[0].chanstart;
			}
			adx->ch[0].type = type_adx03;
	} else if (get32bit(&readbuf[0x10])==0x01F40400) {
			if (adx->ch[0].chanstart-6 < 0x38) adx->ch[0].loop_flag=0; // if header is too small for loop data...
			else {
				adx->ch[0].loop_flag = get32bit(&readbuf[0x24]);
				adx->ch[0].ea = get32bit(&readbuf[0x34]);
				adx->ch[0].sa = get32bit(&readbuf[0x28])*adx->NCH*18/32+adx->ch[0].chanstart;
			}
			adx->ch[0].type = type_adx04;
	} else {
		if (inputfile) {
			CloseHandle(adx->ch[0].infile);
			adx->ch[0].infile=INVALID_HANDLE_VALUE;
		}
		return 1;
	}

	if (adx->ch[0].loop_flag) adx->nrsamples=((adx->ch[0].sa-adx->ch[0].chanstart)+(adx->ch[0].ea-adx->ch[0].sa)*looptimes)*32/adx->NCH/18+masterfadelength*adx->ch[0].sample_rate;

	SetFilePointer(adx->ch[0].infile, adx->ch[0].chanstart, NULL, FILE_BEGIN);
	
    adx->file_length=GetFileSize(adx->ch[0].infile,NULL);

	adx->ch[0].hist1 = 0;
    adx->ch[0].hist2 = 0;
    adx->ch[1].hist1 = 0;
    adx->ch[1].hist2 = 0;

	adx->ch[0].readloc=adx->ch[1].readloc=adx->ch[0].writeloc=adx->ch[1].writeloc=0;

	return 0;
}

void fillbufferADX(CUBEFILE * adx) {
	int i,l;
	short decodebuf[32];
	char ADPCMbuf[18],ADPCMbuf2[18];

	if ((signed long)SetFilePointer(adx->ch[0].infile,0,0,FILE_CURRENT) >= adx->file_length && !adx->ch[0].loop_flag) {
		adx->ch[0].readloc=adx->ch[1].readloc=adx->ch[0].writeloc-1;
		return;
	}

	do {
		if (adx->ch[0].loop_flag && SetFilePointer(adx->ch[0].infile,0,0,FILE_CURRENT) >= adx->ch[0].ea) {
			DisplayError("loop");
			SetFilePointer(adx->ch[0].infile,adx->ch[0].sa,0,FILE_BEGIN);
		}

		l = 0; 
				
		ReadFile(adx->ch[0].infile, ADPCMbuf, 18, &l, NULL);					
		if (l<18) return;
		if (adx->NCH==2) {
			ReadFile(adx->ch[0].infile, ADPCMbuf2, 18, &l, NULL);
			if (l<18) return;
		}
		ADXdecodebuffer(ADPCMbuf,decodebuf, &adx->ch[0].hist1, &adx->ch[0].hist2);

		for(i = 0; i < 32; i++)
			adx->ch[0].chanbuf[adx->ch[0].writeloc++] = decodebuf[i];

		if (adx->NCH==2) {
   			ADXdecodebuffer(ADPCMbuf,decodebuf, &adx->ch[1].hist1, &adx->ch[1].hist2);
			for(i = 0; i < 32; i++)
				adx->ch[1].chanbuf[adx->ch[1].writeloc++] = decodebuf[i];
		}

		if (adx->ch[0].writeloc>=0x8000/8*14) adx->ch[0].writeloc=0;
		if (adx->ch[1].writeloc>=0x8000/8*14) adx->ch[1].writeloc=0;
	} while (adx->ch[0].writeloc != adx->ch[0].readloc);
} 